"""
CSP Common Library

模組化工具集，支援按需安裝：
- csp_lib[mongo]: MongoDB 批次上傳
- csp_lib[redis]: Redis 工具
- csp_lib[modbus]: Modbus 通訊
- csp_lib[can]: CAN Bus 通訊
- csp_lib[all]: 安裝所有功能
"""

__version__ = "0.6.1"
__author__ = "Cheng Sin Pang (鄭善淜)"
__email__ = "donaldpang123@gmail.com"
__license__ = "Apache-2.0"
